import asyncio
from typing import Any

import orjson
import ultima_scraper_api
import websockets
from ultima_scraper_api.apis.background_tasks import BackgroundTask
from websockets.legacy.client import WebSocketClientProtocol

user_types = ultima_scraper_api.user_types

from ultima_scraper_api.classes.make_settings import Config


class DashboardControllerAPI:
    def __init__(self, config: Config) -> None:
        self.listener_args = (
            self.handler,
            config.settings.tui.host,
            config.settings.tui.port,
        )
        self.CONNECTIONS: dict[str, WebSocketClientProtocol] = {}
        self.background_task = BackgroundTask()
        self.background_task.create_background_task(self.start_websocket)
        _task = asyncio.create_task(self.worker())
        self.prompt_queue: asyncio.Queue[Any] = asyncio.Queue()
        _blocking = self.wait_for_connection()

    async def start_websocket(self, kwargs: dict[str, Any] = {}):
        await self.listener()

    async def listener(self):
        async with websockets.serve(*self.listener_args):  # type: ignore
            await asyncio.Future()

    def wait_for_connection(self):
        found_connections = False
        while not found_connections:
            if self.CONNECTIONS:
                found_connections = True
        return found_connections

    async def handler(self, websocket: WebSocketClientProtocol):
        self.CONNECTIONS[websocket.id.hex] = websocket
        _reader_task = asyncio.create_task(self.reader(websocket))
        try:
            await websocket.wait_closed()
        finally:
            self.CONNECTIONS.pop(websocket.id.hex)

    async def reader(self, websocket: WebSocketClientProtocol):
        # Server crashes when GUI closes on a prompt/or is waiting for response
        async for message in websocket:
            data = orjson.loads(message)
            match data["type"]:
                case "prompt":
                    await self.prompt_queue.put(data)
                case _:
                    pass

    async def worker(self):
        while True:
            _item = await self.background_task.queue.get()
            pass

    async def message_all(self, data: Any):
        self.wait_for_connection()
        websockets.broadcast([*self.CONNECTIONS.values()], data)  # type: ignore

    async def prompt(self, string: str) -> str:
        data = orjson.dumps({"type": "prompt", "value": string})
        await self.message_all(data)
        # old_connections = self.CONNECTIONS.copy()
        while True:
            if self.prompt_queue.qsize():
                response = await self.prompt_queue.get()
                return str(response["value"])
            # If no clients, we'll retry prompt
            # Still need to find a way to send the prompt to connections that join after
            if not self.CONNECTIONS:
                return await self.prompt(string)

    async def change_title(self, string: str):
        data = orjson.dumps({"type": "change_title", "value": string})
        await self.message_all(data)

    async def datatable_monitor(self, subscription_list: list[user_types]):
        while True:
            data_byte_array: list[str] = [
                x.convert_to_dill().hex() for x in subscription_list
            ]
            data = orjson.dumps({"type": "datatable_monitor", "value": data_byte_array})
            await self.message_all(data)
            await asyncio.sleep(1)

    # async def convert_object_to_dill(self, parent: dict[str, Any], key: str, data: Any):
    #     test = parent.__dict__.copy()
    #     test["dashboard_controller_api"] = None
    #     new = []
    #     for x in data:
    #         if not isinstance(x, str):
    #             x = x.convert_to_dill()
    #         new.append(x)
    #         pass
    #     test[key] = new
    #     ok: str = dill.dumps(test).hex()
    #     return ok



import asyncio
from datetime import datetime
from typing import Any

from websockets import client

from ultima_scraper.managers.dashboard_controller_api import DashboardControllerAPI


class UiManager:
    # Will use this to display things to the terminal in the future
    def __init__(self) -> None:
        self.mode: str = "standard"
        self.dashboard_controller_api: DashboardControllerAPI | None = None

    async def display(self, data: Any, mode: str = ""):
        mode = mode if mode else self.mode
        match mode:
            case "standard":
                print(f"[{datetime.now().replace(microsecond=0)}] {data}")
            case "dashboard":
                if self.dashboard_controller_api:
                    intask = self.dashboard_controller_api.datatable_monitor(data)
                    _task = asyncio.create_task(intask)
                pass
            case _:
                pass

    async def websocket_display(self):
        async with client.connect("ws://localhost:1111/scrape/ws") as websocket:
            while True:
                message = await websocket.recv()
                if isinstance(message, str):
                    print(message)
                elif isinstance(message, dict):
                    print(message.decode())



from multiprocessing import Process
from typing import TYPE_CHECKING, Any

import requests
import uvicorn
from fastapi import (
    APIRouter,
    Depends,
    FastAPI,
    HTTPException,
    Request,
    Response,
    WebSocket,
)
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel

if TYPE_CHECKING:
    from ultima_scraper.ultima_scraper import UltimaScraper


def p_function(fast_api: FastAPI, port: int):
    uvicorn.run(  # type: ignore
        fast_api,
        host="0.0.0.0",
        port=port,
        # log_level="debug",
    )


def activate_api(fast_api: FastAPI, port: int):
    origins = [
        "*",
    ]
    fast_api.add_middleware(
        CORSMiddleware,
        allow_origins=origins,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    server = Process(
        target=p_function,
        args=(
            fast_api,
            port,
        ),
        daemon=True,
    )
    server.start()
    while True:
        try:
            requests.get(f"http://localhost:{port}")
            break
        except requests.exceptions.ConnectionError:
            continue
    return server


class App(FastAPI):
    ultima_scraper: "UltimaScraper"

    def __init__(
        self,
        ultima_scraper: "UltimaScraper",
        *args: tuple[Any],
        **kwargs: dict[str, Any],
    ):
        super().__init__(*args, **kwargs)
        self.counter = 0
        self.ultima_scraper = ultima_scraper
        App.ultima_scraper = ultima_scraper
        self.include_router(router)


router = APIRouter(
    prefix="/scrape",
    responses={404: {"description": "Not found"}},
)


@router.websocket("/ws")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()
    while True:
        await websocket.send_text(f"Message text was: {'data'}")


class ScrapeAndDownloadOptions(BaseModel):
    authed_identifiers: list[int | str] = []
    site_name: str
    username: str


@router.get("/")
async def root(options: ScrapeAndDownloadOptions):
    ultima_scraper = App.ultima_scraper
    site_name = options.site_name
    async with ultima_scraper.ultima_archive_db_api.get_site_api(
        site_name
    ) as db_site_api:
        performer = await ultima_scraper.prepared_performer(
            site_name=site_name,
            identifier=options.username,
            buyer_identifiers=options.authed_identifiers,
        )
        if performer:
            db_performer = await db_site_api.get_user(performer.id, load_content=True)
            datascraper = ultima_scraper.datascraper_manager.find_datascraper(site_name)
            assert datascraper
            db_site_api.datascraper = datascraper
            metadata_manager = await ultima_scraper.prepare_filesystem(performer)
            api = performer.get_api()
            job_manager = api.job_manager
            local_jobs: list[Any] = []
            jobs = job_manager.create_jobs(
                "Scrape",
                # content_options.final_choices,
                ["Posts"],
                datascraper.prepare_scraper,
                [performer, metadata_manager],
            )
            local_jobs.extend(jobs)
            jobs = job_manager.create_job(
                "DatabaseImport",
                db_site_api.update_user,
                [performer, db_performer],
            )
            local_jobs.extend([jobs])
            performer.jobs.extend(local_jobs)
            for local_job in local_jobs:
                job_manager.queue.put_nowait(local_job)
            await job_manager.process_jobs()
        else:
            return {"status": False}
        return {"status": True}


@router.get("/download")
async def download(options: ScrapeAndDownloadOptions):
    ultima_scraper = App.ultima_scraper
    site_name = options.site_name
    datascraper = ultima_scraper.datascraper_manager.find_datascraper(site_name)
    assert datascraper
    performers = datascraper.api.find_user(options.username)
    for performer in performers:
        api = performer.get_api()
        job_manager = api.job_manager
        jobs = job_manager.create_jobs(
            "Download",
            # content_options.final_choices,
            ["Posts"],
            datascraper.prepare_downloads,
            [performer],
        )
        job_manager.queue.put_nowait(jobs[0])
        performer.jobs.extend(jobs)
        await job_manager.process_jobs()
    return {"status": True}


@router.get("/active_user")
async def active_user():
    ultima_scraper = App.ultima_scraper
    active_user_value = ultima_scraper.active_user.value
    final_value = active_user_value if active_user_value != 0 else None
    return {"active_user": final_value}



import asyncio
import multiprocessing
import threading
import timeit
from contextlib import asynccontextmanager
from ctypes import c_int
from datetime import datetime, timedelta
from multiprocessing.managers import SyncManager
from multiprocessing.sharedctypes import SynchronizedBase
from pathlib import Path
from typing import Any

import ultima_scraper_api
import ultima_scraper_api.helpers.main_helper as main_helper
import ultima_scraper_collection.managers.datascraper_manager.datascrapers.fansly as m_fansly
import ultima_scraper_collection.managers.datascraper_manager.datascrapers.onlyfans as m_onlyfans
from sqlalchemy import select
from sqlalchemy.orm import joinedload
from ultima_scraper_api import SITE_LITERALS, SUPPORTED_SITES
from ultima_scraper_api.apis.onlyfans.classes.auth_model import OnlyFansAuthModel
from ultima_scraper_api.apis.onlyfans.classes.only_drm import OnlyDRM
from ultima_scraper_api.managers.job_manager.jobs.custom_job import CustomJob
from ultima_scraper_collection.config import GlobalAPI, UltimaScraperCollectionConfig
from ultima_scraper_collection.managers.datascraper_manager.datascraper_manager import (
    DataScraperManager,
)
from ultima_scraper_collection.managers.metadata_manager.metadata_manager import (
    MetadataManager,
)
from ultima_scraper_collection.managers.option_manager import OptionManager
from ultima_scraper_collection.managers.server_manager import ServerManager
from ultima_scraper_collection.projects.project_manager import Project
from ultima_scraper_db.databases.ultima_archive import merged_metadata as archive_m
from ultima_scraper_db.databases.ultima_archive.database_api import ArchiveAPI
from ultima_scraper_db.databases.ultima_archive.schemas.templates.site import (
    BoughtContentModel,
    ContentMediaAssoModel,
    JobModel,
    MassMessageModel,
    MassMessageStatModel,
    MessageModel,
    NotificationModel,
    PostModel,
    RemoteURLModel,
    SubscriptionModel,
    UserModel,
)
from ultima_scraper_db.databases.ultima_archive.site_api import SiteAPI
from ultima_scraper_db.managers.database_manager import Alembica

from ultima_scraper.api import App
from ultima_scraper.managers.ui_manager import UiManager

api_types = ultima_scraper_api.api_types
auth_types = ultima_scraper_api.auth_types
user_types = ultima_scraper_api.user_types


class UltimaScraper:
    def __init__(
        self,
        config: UltimaScraperCollectionConfig,
        rest_api: App | None = None,
        process_manager: SyncManager | None = None,
        update_db: bool = False,
    ) -> None:
        self.ui_manager = UiManager()
        self.rest_api = rest_api
        self.option_manager = OptionManager()
        self.config = config
        if process_manager:
            self.active_user = process_manager.Value(c_int, 0)
            self.active_download = process_manager.dict()
            self.misc = process_manager.dict()
        self.update_db = update_db

    async def __aenter__(self):
        await self.init(self.update_db)
        return self

    async def __aexit__(self, exc_type: None, exc_value: None, traceback: None):
        if self.server_manager:
            await self.server_manager.ultima_archive_db_api.management_schema.session.aclose()

            for (
                site_api
            ) in self.server_manager.ultima_archive_db_api.site_apis.values():
                await site_api.schema.session.aclose()

    async def init(self, update_db: bool = False):
        project = Project("UltimaArchiver")

        db_info = self.config.settings.databases[0].connection_info.dict()
        generate = migrate = False
        if update_db:
            generate = migrate = True
        ultima_archive_db = await project._init_db(
            db_info, Alembica(generate=generate, migrate=migrate), archive_m
        )
        self.ultima_archive_db_api = await ArchiveAPI(ultima_archive_db).init()
        self.server_manager = self.ultima_archive_db_api.server_manager
        self.datascraper_manager = DataScraperManager(
            self.ultima_archive_db_api.server_manager, self.config
        )

        # self.ultima_archive_db_api.activate_api(
        #     self.ultima_archive_db_api.fast_api, 2140
        # )

    async def process_server_jobs(
        self,
        site_name: SITE_LITERALS,
        filter_by_user_ids: list[int] = [],
        filter_by_buyer_ids: list[int] = [],
        skip_user_ids: list[int] = [],
    ):
        async with self.ultima_archive_db_api.get_site_api(site_name) as site_db_api:
            downloaded_count = 0
            server_manager = self.server_manager
            server_jobs = await site_db_api.get_jobs(
                server_manager.active_server.id,
                category="download",
                active=True,
                limit=None,
            )

            datascraper = self.datascraper_manager.select_datascraper(
                site_name=site_name
            )

            assert datascraper
            site_db_api.datascraper = datascraper
            for server_job in server_jobs:
                if filter_by_user_ids and server_job.user_id not in filter_by_user_ids:
                    continue
                if skip_user_ids and server_job.user_id in skip_user_ids:
                    continue
                server_job = await self.process_server_job(
                    site_name,
                    server_job,
                    site_db_api,
                    filter_by_buyer_ids=filter_by_buyer_ids,
                )
                if not server_job.active:
                    downloaded_count += 1

    async def process_server_job(
        self,
        site_name: SITE_LITERALS,
        server_job: JobModel,
        site_db_api: SiteAPI,
        filter_by_buyer_ids: list[int] = [],
    ):
        datascraper = site_db_api.datascraper
        assert datascraper
        content_types = datascraper.api.CategorizedContent()
        content_types_keys = content_types.get_keys()
        job_manager = datascraper.api.job_manager
        downloaded_public_content = downloaded_paid_content = False
        db_performer: UserModel | None = None
        async for performer, db_buyer in self.prepared_performer(
            site_name=site_name,
            identifier=server_job.user_id,
            buyer_identifiers=filter_by_buyer_ids,
        ):
            print(
                f"Performer: {performer.username} ({performer.id}) | Authed: {db_buyer.username} ({db_buyer.id})"
            )
            self.active_user.value = performer.id
            if not db_performer:
                db_performer = await site_db_api.get_user(
                    server_job.user_id, load_content=True, load_media=True
                )
                assert db_performer
            authed = performer.get_authed()
            if performer.is_authed_user() and isinstance(authed, OnlyFansAuthModel):
                w9_bytes = await authed.user.get_w9_form()
                if w9_bytes:
                    w9_path = Path(
                        "__user_data__/w9_forms/",
                        f"{authed.user.username} ({authed.user.id}).pdf",
                    )
                    w9_path.parent.mkdir(parents=True, exist_ok=True)
                    w9_path.write_bytes(w9_bytes)

            db_subscription = await db_buyer.find_subscription(db_performer.id)
            metadata_manager = datascraper.find_metadata_manager(db_performer.id)
            content_options = await datascraper.option_manager.create_option(
                content_types_keys, "contents", True
            )
            local_jobs: list[CustomJob] = []
            job_set = job_manager.create_jobs(
                "Scrape",
                ["Messages"],
                datascraper.scrape_vault,
                [performer, db_performer],
            )

            final_content_options = content_options.final_choices
            # downloaded_public_content = True
            if downloaded_public_content:
                removing = ["Stories", "Posts", "Highlights"]
                final_content_options = list(
                    filter(lambda x: x not in removing, final_content_options)
                )
            final_download_options = ["Uncategorized", *final_content_options]

            if not performer.is_authed_user():
                final_download_options.remove("Uncategorized")

            job_set_2 = job_manager.create_jobs(
                "Scrape",
                final_content_options,
                datascraper.prepare_scraper,
                [performer, metadata_manager],
            )
            job_set_3 = job_manager.create_job(
                "DatabaseImport",
                site_db_api.update_user,
                [performer, db_performer],
            )
            job_set_4 = job_manager.create_jobs(
                "Download",
                final_download_options,
                datascraper.prepare_downloads,
                [performer, db_performer],
            )
            local_jobs.extend(job_set + job_set_2 + [job_set_3] + job_set_4)
            performer.jobs.extend(local_jobs)
            for local_job in local_jobs:
                job_manager.queue.put_nowait(local_job)
            await datascraper.datascraper.api.job_manager.process_jobs()
            if all(job.done for job in performer.jobs):
                fmu = datascraper.filesystem_manager.get_file_manager(performer.id)
                await fmu.cleanup()
                downloaded_public_content = downloaded_paid_content = True
            else:
                db_subscription = await db_buyer.find_subscription(db_performer.id)
                if db_subscription:
                    if db_subscription.expires_at > datetime.now().astimezone():
                        downloaded_public_content = True
                        db_subscription.downloaded_at = datetime.now()
                    if db_subscription.paid_content:
                        downloaded_paid_content = True
                else:
                    downloaded_paid_content = True
        if server_job.skippable or downloaded_public_content or downloaded_paid_content:
            server_job.active = False
            server_job.completed_at = datetime.now()
        await site_db_api.get_session().commit()
        return server_job

    async def prepared_performer(
        self,
        site_name: SITE_LITERALS,
        identifier: int | str,
        buyer_identifiers: list[int | str] = [],
    ):
        async with self.ultima_archive_db_api.get_site_api(site_name) as db_site_api:
            db_user = await db_site_api.get_user(identifier)
            if not db_user:
                return
            db_buyers = await db_user.find_buyers(
                active=None if db_user.favorite else True,
                identifiers=buyer_identifiers,
                active_user=True,
            )
            if not db_buyers:
                return

            datascraper = self.datascraper_manager.find_datascraper(site_name)
            assert datascraper
            site_api = datascraper.api
            db_performer = await db_site_api.get_user(identifier, load_aliases=True)
            assert db_performer
            for db_buyer in db_buyers:
                db_auth = db_buyer.find_auth()
                if not db_auth:
                    continue
                auth_details = db_auth.convert_to_auth_details(site_name)
                authed = await site_api.login(auth_details.export())
                if not authed:
                    await db_auth.deactivate()
                    continue
                await db_auth.activate()
                authed.drm = OnlyDRM(
                    Path("__user_data__/drm_device/device_client_id_blob"),
                    Path("__user_data__/drm_device/device_private_key"),
                    authed,
                )
                performer = await datascraper.get_performer(authed, db_performer)
                if not performer:
                    print(
                        f"Performer: {db_performer.username} ({db_performer.id}) not found."
                    )
                    continue
                performer.aliases = [x.username for x in db_performer.aliases]
                await datascraper.prepare_filesystem(performer)
                datascraper.resolve_content_manager(performer)
                yield performer, db_buyer

    async def start(
        self,
        site_name: str,
        api_: api_types | None = None,
    ):
        archive_time = timeit.default_timer()

        datascraper = self.datascraper_manager.select_datascraper(
            site_name,
        )
        if datascraper:
            datascraper.filesystem_manager.activate_directory_manager(
                datascraper.site_config
            )
            await self.start_datascraper(datascraper)
        stop_time = str(int(timeit.default_timer() - archive_time) / 60)[:4]
        await self.ui_manager.display(f"Archive Completed in {stop_time} Minutes")
        return api_

    async def start_datascraper(
        self,
        datascraper: m_onlyfans.OnlyFansDataScraper | m_fansly.FanslyDataScraper,
    ):
        api = datascraper.api
        if datascraper.filesystem_manager.directory_manager:
            datascraper.filesystem_manager.directory_manager.create_directories()
        site_config = datascraper.site_config
        await self.process_profiles(api, self.config.settings)
        scrapable_users: list[user_types] = []
        auth_count = 0
        profile_options = await self.option_manager.create_option(
            api.auths, "profiles", site_config.auto_profile_choice
        )
        api.auths = profile_options.final_choices
        # await dashboard_controller.update_main_table(api)
        identifiers = []
        if site_config.auto_performer_choice:
            subscription_options = await self.option_manager.create_option(
                scrapable_users, "subscriptions", site_config.auto_performer_choice
            )
            if not subscription_options.scrape_all():
                identifiers = subscription_options.return_auto_choice()
            self.option_manager.performer_options = subscription_options
        for auth in api.auths:
            auth: auth_types = auth
            if not auth.get_auth_details():
                continue
            setup = False
            setup, _subscriptions = await datascraper.account_setup(
                auth, site_config, identifiers
            )
            if not setup:
                auth_details: dict[str, Any] = {}
                auth_details["auth"] = auth.get_auth_details().export()
                profiles_directory = datascraper.filesystem_manager.profiles_directory
                _user_auth_filepath = profiles_directory.joinpath(
                    api.site_name, auth.get_auth_details().username, "auth.json"
                )
                # main_helper.export_json(auth_details, user_auth_filepath)
                continue
            auth_count += 1
            scrapable_users.extend(await auth.get_scrapable_users())
            # Do stuff with authed user
            if not auth.drm:
                device_client_id_blob_path = (
                    datascraper.filesystem_manager.devices_directory.joinpath(
                        "device_client_id_blob"
                    )
                )
                device_private_key_path = (
                    datascraper.filesystem_manager.devices_directory.joinpath(
                        "device_private_key"
                    )
                )
                if (
                    device_client_id_blob_path.exists()
                    and device_private_key_path.exists()
                ):
                    auth.drm = OnlyDRM(
                        device_client_id_blob_path,
                        device_private_key_path,
                        auth,
                    )
        await api.remove_invalid_auths()
        subscription_options = await self.option_manager.create_option(
            scrapable_users, "subscriptions", site_config.auto_performer_choice
        )
        self.option_manager.subscription_options = subscription_options
        final_job_user_list = await datascraper.configure_datascraper_jobs()
        await self.assign_jobs(final_job_user_list)
        await datascraper.datascraper.api.job_manager.process_jobs()
        # if global_settings.helpers.delete_empty_directories:
        #     for job_user in job_user_list:
        #         await main_helper.delete_empty_directories(
        #             job_user.directory_manager.user.download_directory,
        #             datascraper.api.filesystem_manager,
        #         )

    async def process_profiles(
        self,
        api: api_types,
        global_settings: UltimaScraperCollectionConfig.Settings,
    ):
        from ultima_scraper_collection.managers.filesystem_manager import (
            FilesystemManager,
        )

        site_name = api.site_name
        filesystem_manager = FilesystemManager()
        profile_directory = filesystem_manager.profiles_directory.joinpath(site_name)
        profile_directory.mkdir(parents=True, exist_ok=True)
        temp_users = list(filter(lambda x: x.is_dir(), profile_directory.iterdir()))
        temp_users = filesystem_manager.remove_mandatory_files(temp_users)
        for user_profile in temp_users:
            user_auth_filepath = user_profile.joinpath("auth.json")
            temp_json_auth = main_helper.import_json(user_auth_filepath)
            json_auth = temp_json_auth.get("auth", {})
            if not json_auth.get("active", None):
                continue
            json_auth["username"] = user_profile.name
            authed = await api.login(json_auth)
            authed.session_manager.add_proxies(global_settings.network.proxies)
            if authed.is_authed():
                site_db_api = self.ultima_archive_db_api.get_site_api(
                    authed.get_api().site_name
                )
                datascraper = (
                    site_db_api.datascraper
                ) = self.datascraper_manager.active_datascraper
                assert datascraper
                await datascraper.filesystem_manager.create_directory_manager(
                    datascraper, authed.user
                )
                found_db_user = await site_db_api.get_user(authed.id)
                await site_db_api.update_user(authed.user, found_db_user)
                pass
            datas = {"auth": authed.get_auth_details().export()}
            if datas:
                main_helper.export_json(datas, user_auth_filepath)
        return api

    async def assign_jobs(self, user_list: set[user_types]):
        datascraper = self.datascraper_manager.active_datascraper
        if not datascraper:
            return
        await self.ui_manager.display("Assigning Jobs")
        filesystem_manager = datascraper.filesystem_manager
        JBM = datascraper.api.job_manager
        site_config = datascraper.site_config
        content_types = datascraper.api.ContentTypes()
        content_types_keys = content_types.get_keys()
        media_types = datascraper.api.MediaTypes()
        media_types_keys = media_types.get_keys()

        for user in user_list:
            await filesystem_manager.create_directory_manager(datascraper, user)
            await filesystem_manager.format_directories(user)
            metadata_manager = MetadataManager(user, filesystem_manager)
            await metadata_manager.process_legacy_metadata()
            datascraper.metadata_manager_users[user.id] = metadata_manager
            authed = user.get_authed()
            site_db = await self.server_manager.get_site_db(
                authed.get_api().site_name, datascraper
            )
            found_db_user = await site_db.get_user(user.id)

            local_jobs: list[CustomJob] = []
            auto_content_choice = (
                site_config.auto_content_choice
                if not user.scrape_whitelist
                else user.scrape_whitelist
            )

            content_options = await self.option_manager.create_option(
                content_types_keys, "contents", auto_content_choice
            )
            jobs = JBM.create_jobs(
                "Scrape",
                content_options.final_choices,
                datascraper.prepare_scraper,
                [user, metadata_manager],
            )
            local_jobs.extend(jobs)

            jobs = JBM.create_job(
                "DatabaseImport",
                site_db.update_user,
                [user, found_db_user],
            )
            local_jobs.append(jobs)

            jobs = JBM.create_jobs(
                "Download",
                content_options.final_choices,
                datascraper.prepare_downloads,
                [user],
            )
            local_jobs.extend(jobs)

            user.jobs.extend(local_jobs)

            media_options = await self.option_manager.create_option(
                media_types_keys, "medias", site_config.auto_media_choice
            )
            JBM.add_media_type_to_jobs(media_options.final_choices)

            for local_job in local_jobs:
                JBM.queue.put_nowait(local_job)
            await asyncio.sleep(0)
        pass
